@@ -25,8 +25,8 @@
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <CL/opencl.h>
-#include "poclu.h"
+
+#include "pocl_opencl.h"
 
 #ifdef _WIN32
 #  include "vccompat.hpp"
@@ -57,7 +57,7 @@ main (int argc, char **argv)
   spir = (argc > 1 && argv[1][0] == 's');
   spirv = (argc > 1 && argv[1][0] == 'v');
   poclbin = (argc > 1 && argv[1][0] == 'b');
-  const char *explicit_binary_path = (poclbin && (argc > 2)) ? argv[2] : NULL;
+  const char *explicit_binary_path = (argc > 2) ? argv[2] : NULL;
 
   const char *basename = "example2a";
   err = poclu_load_program (context, device, basename, spir, spirv, poclbin,
