@@ -21,20 +21,20 @@
    IN THE SOFTWARE.
 */
 
-/* For srandom. */
+// For srandom
 #define _DEFAULT_SOURCE
 #define _BSD_SOURCE
 
-#include <CL/opencl.h>
 #include <assert.h>
 #include <errno.h>
 #include <math.h>
-#include <poclu.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 
+#include "pocl_opencl.h"
+
 #ifdef _WIN32
 #  include "vccompat.hpp"
 #endif
