@@ -28,6 +28,81 @@
 #include "pocl_util.h"
 #include "pocl_shared.h"
 
+/* max number of lines in output of 'llvm-spirv --spec-const-info' */
+#define MAX_SPEC_CONSTANT_LINES 4096
+/* max bytes in output of 'llvm-spirv --spec-const-info' */
+#define MAX_OUTPUT_BYTES 65536
+
+#ifdef ENABLE_SPIRV
+static int
+get_program_spec_constants (cl_program program, char *program_bc_spirv)
+{
+  char *args[] = { LLVM_SPIRV, "--spec-const-info", program_bc_spirv, NULL };
+  char captured_output[MAX_OUTPUT_BYTES];
+  size_t captured_bytes = MAX_OUTPUT_BYTES;
+  int errcode = CL_SUCCESS;
+
+  errcode = pocl_run_command_capture_output (captured_output, &captured_bytes,
+                                             args);
+  POCL_RETURN_ERROR_ON ((errcode != 0), CL_INVALID_BINARY, "External command "
+                        "(llvm-spirv --spec-const-info) failed!\n");
+
+  captured_output[captured_bytes] = 0;
+  char *lines[MAX_SPEC_CONSTANT_LINES];
+  unsigned num_lines = 0;
+  char delim[2] = { 0x0A, 0x0 };
+  char *token = strtok (captured_output, delim);
+  while (num_lines < MAX_SPEC_CONSTANT_LINES && token != NULL)
+    {
+      lines[num_lines++] = strdup (token);
+      token = strtok (NULL, delim);
+    }
+  POCL_GOTO_ERROR_ON ((num_lines == 0 || num_lines >= MAX_SPEC_CONSTANT_LINES),
+                      CL_INVALID_BINARY,
+                      "Can't parse output from llvm-spirv\n");
+
+  unsigned num_const = 0;
+  int r = sscanf (
+      lines[0], "Number of scalar specialization constants in the module = %u",
+      &num_const);
+  POCL_GOTO_ERROR_ON ((r < 1 || num_const > num_lines), CL_INVALID_BINARY,
+                      "Can't parse first line of output");
+
+  program->num_spec_consts = num_const;
+  if (num_const > 0)
+    {
+      program->spec_const_ids = calloc (num_const, sizeof (cl_uint));
+      program->spec_const_sizes = calloc (num_const, sizeof (cl_uint));
+      program->spec_const_values = calloc (num_const, sizeof (uint64_t));
+      program->spec_const_is_set = calloc (num_const, sizeof (char));
+      for (unsigned i = 0; i < program->num_spec_consts; ++i)
+        {
+          unsigned spec_id, spec_size;
+          int r
+              = sscanf (lines[i + 1], "Spec const id = %u, size in bytes = %u",
+                        &spec_id, &spec_size);
+          POCL_GOTO_ERROR_ON ((r < 2), CL_INVALID_BINARY,
+                              "Can't parse %u-th line of output:\n%s\n",
+                              i+1, lines[i+1]);
+          program->spec_const_ids[i] = spec_id;
+          program->spec_const_sizes[i] = spec_size;
+          program->spec_const_values[i] = 0;
+          program->spec_const_is_set[i] = CL_FALSE;
+        }
+    }
+  errcode = CL_SUCCESS;
+ERROR:
+  for (unsigned i = 0; i < num_lines; ++i)
+    free (lines[i]);
+  if (errcode != CL_SUCCESS)
+    {
+      program->num_spec_consts = 0;
+    }
+  return errcode;
+}
+#endif
+
+
 CL_API_ENTRY cl_program CL_API_CALL
 POname(clCreateProgramWithIL)(cl_context context,
                               const void *il,
@@ -35,18 +110,16 @@ POname(clCreateProgramWithIL)(cl_context context,
                               cl_int *errcode_ret)
 CL_API_SUFFIX__VERSION_2_1
 {
-  /* SPIR is disabled when building with conformance */
-#ifdef ENABLE_CONFORMANCE
+  /* if SPIR-V is disabled, return error early */
+#if defined(ENABLE_CONFORMANCE) && !defined(ENABLE_SPIRV)
   if (errcode_ret)
     *errcode_ret = CL_INVALID_OPERATION;
   return NULL;
 #endif
   cl_program program = NULL;
   int errcode = CL_SUCCESS;
-#ifdef ENABLE_SPIRV
-  uint64_t converted_spir_file_size = 0;
-  char *converted_spir_file = NULL;
-#endif
+  char program_bc_spirv[POCL_FILENAME_LENGTH];
+  program_bc_spirv[0] = 0;
 
   POCL_GOTO_ERROR_COND ((context == NULL), CL_INVALID_CONTEXT);
 
@@ -71,36 +144,6 @@ CL_API_SUFFIX__VERSION_2_1
       "The IL provided to clCreateProgramWithIL "
       "is not recognized as SPIR-V!\n");
 
-#ifdef ENABLE_SPIRV
-  if (is_spirv_kernel)
-    {
-      /* convert and the SPIR-V to LLVM IR with spir triple */
-      POCL_MSG_PRINT_LLVM (
-          "SPIR-V binary detected, converting to LLVM SPIR\n");
-
-      char program_bc_spirv[POCL_FILENAME_LENGTH];
-      char program_bc_temp[POCL_FILENAME_LENGTH];
-      pocl_cache_write_spirv (program_bc_spirv, (const char *)il,
-                              (uint64_t)length);
-      pocl_cache_tempname (program_bc_temp, ".bc", NULL);
-
-      char *args[] = { LLVM_SPIRV,       "-r", "-o", program_bc_temp,
-                       program_bc_spirv, NULL };
-
-      errcode = pocl_run_command (args);
-      POCL_GOTO_ERROR_ON (
-          (errcode != 0), CL_INVALID_VALUE,
-          "External command (llvm-spirv translator) failed!\n");
-
-      /* load LLVM SPIR binary. */
-      pocl_read_file (program_bc_temp, &converted_spir_file,
-                      &converted_spir_file_size);
-      POCL_GOTO_ERROR_ON ((converted_spir_file == NULL), CL_INVALID_VALUE,
-                          "Can't read converted bitcode file\n");
-      pocl_remove (program_bc_temp);
-    }
-#endif
-
   /* TODO should we create a program for all devices ?
    * should we fail if we can't create for all devices ?
    * this seems to be unspecified.
@@ -109,12 +152,8 @@ CL_API_SUFFIX__VERSION_2_1
    * and fail if there's no such device in context. */
   unsigned num = context->num_devices;
   unsigned num_devices_with_spir = 0;
-
   cl_device_id *devices_with_spir
       = (cl_device_id *)alloca (num * sizeof (cl_device_id));
-  size_t *devices_with_spir_lens = (size_t *)alloca (num * sizeof (size_t));
-  const unsigned char **devices_with_spir_bins
-      = (const unsigned char **)alloca (num * sizeof (unsigned char *));
 
   for (unsigned i = 0; i < num; ++i)
     {
@@ -124,46 +163,38 @@ CL_API_SUFFIX__VERSION_2_1
       if (dev->ops->supports_binary (dev, length, il))
         {
           devices_with_spir[num_devices_with_spir] = dev;
-          devices_with_spir_lens[num_devices_with_spir] = length;
-          devices_with_spir_bins[num_devices_with_spir] = il;
           ++num_devices_with_spir;
         }
-#ifdef ENABLE_SPIRV
-      else if (dev->ops->supports_binary (dev, converted_spir_file_size,
-                                          converted_spir_file))
-        {
-          devices_with_spir[num_devices_with_spir] = dev;
-          devices_with_spir_lens[num_devices_with_spir]
-              = (size_t)converted_spir_file_size;
-          devices_with_spir_bins[num_devices_with_spir]
-              = (const unsigned char *)converted_spir_file;
-          ++num_devices_with_spir;
-        }
-#endif
     }
 
   POCL_GOTO_ERROR_ON ((num_devices_with_spir == 0), CL_INVALID_OPERATION,
                       "No device in context supports SPIR\n");
-  POCL_MSG_PRINT_GENERAL ("Creating context from IL for %u devices\n",
-                          num_devices_with_spir);
 
-  program = POname (clCreateProgramWithBinary) (
-      context, num_devices_with_spir, devices_with_spir,
-      devices_with_spir_lens, devices_with_spir_bins, NULL, &errcode);
-  if (errcode == CL_SUCCESS)
-    {
-      POCL_LOCK_OBJ (program);
-      program->program_il = (char *)malloc (length);
-      memcpy (program->program_il, il, length);
-      program->program_il_size = length;
-      POCL_UNLOCK_OBJ (program);
-    }
+  program = create_program_skeleton (context, num_devices_with_spir,
+                                     devices_with_spir, NULL, NULL, NULL,
+                                     &errcode, CL_TRUE);
+  if (program == NULL)
+    goto ERROR;
+
+  /* save the IL into cl_program, and find out spec constants */
+  program->program_il = (char *)malloc (length);
+  memcpy (program->program_il, il, length);
+  program->program_il_size = length;
+#ifdef ENABLE_SPIRV
+  errcode = pocl_cache_write_spirv (program_bc_spirv, (const char *)il,
+                                    (uint64_t)length);
+  get_program_spec_constants (program, program_bc_spirv);
+#endif
 
 ERROR:
   if (errcode_ret)
     *errcode_ret = errcode;
 #ifdef ENABLE_SPIRV
-  POCL_MEM_FREE (converted_spir_file);
+  if (pocl_get_bool_option("POCL_LEAVE_KERNEL_COMPILER_TEMP_FILES", 0) == 0)
+    {
+      if (program_bc_spirv[0] != 0)
+        pocl_remove (program_bc_spirv);
+    }
 #endif
   return program;
 }
