@@ -46,51 +46,12 @@ POname(clEnqueueCopyBufferRect)(cl_command_queue command_queue,
   POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (command_queue)),
                           CL_INVALID_COMMAND_QUEUE);
 
-  cl_int err = pocl_rect_copy (
-    command_queue,
-    CL_COMMAND_COPY_BUFFER_RECT,
-    src_buffer, CL_FALSE,
-    dst_buffer, CL_FALSE,
-    src_origin, dst_origin, region,
-    src_row_pitch, src_slice_pitch,
-    dst_row_pitch, dst_slice_pitch,
-    num_events_in_wait_list, event_wait_list,
-    event,
-    &cmd);
-
-  if (err != CL_SUCCESS)
-    return err;
-
-  size_t src_offset = 0;
-  POCL_CONVERT_SUBBUFFER_OFFSET (src_buffer, src_offset);
-
-  size_t dst_offset = 0;
-  POCL_CONVERT_SUBBUFFER_OFFSET (dst_buffer, dst_offset);
-
-  POCL_RETURN_ERROR_ON((src_buffer->size > command_queue->device->max_mem_alloc_size),
-                        CL_OUT_OF_RESOURCES,
-                        "src is larger than device's MAX_MEM_ALLOC_SIZE\n");
-
-  POCL_RETURN_ERROR_ON((dst_buffer->size > command_queue->device->max_mem_alloc_size),
-                        CL_OUT_OF_RESOURCES,
-                        "src is larger than device's MAX_MEM_ALLOC_SIZE\n");
-
-  cl_device_id dev = command_queue->device;
-
-  cmd->command.copy_rect.src_mem_id = &src_buffer->device_ptrs[dev->global_mem_id];
-  cmd->command.copy_rect.src = src_buffer;
-  cmd->command.copy_rect.dst_mem_id = &dst_buffer->device_ptrs[dev->global_mem_id];
-  cmd->command.copy_rect.dst = dst_buffer;
-
-  cmd->command.copy_rect.src_origin[0] = src_offset + src_origin[0];
-  cmd->command.copy_rect.src_origin[1] = src_origin[1];
-  cmd->command.copy_rect.src_origin[2] = src_origin[2];
-  cmd->command.copy_rect.dst_origin[0] = dst_offset + dst_origin[0];
-  cmd->command.copy_rect.dst_origin[1] = dst_origin[1];
-  cmd->command.copy_rect.dst_origin[2] = dst_origin[2];
-  cmd->command.copy_rect.region[0] = region[0];
-  cmd->command.copy_rect.region[1] = region[1];
-  cmd->command.copy_rect.region[2] = region[2];
+  cl_int errcode = pocl_copy_buffer_rect_common (
+      NULL, command_queue, src_buffer, dst_buffer, src_origin, dst_origin,
+      region, src_row_pitch, src_slice_pitch, dst_row_pitch, dst_slice_pitch,
+      num_events_in_wait_list, event_wait_list, event, NULL, NULL, &cmd);
+  if (errcode != CL_SUCCESS)
+    return errcode;
 
   pocl_command_enqueue (command_queue, cmd);
 
