@@ -40,8 +40,9 @@ POname(clEnqueueSVMMemFill) (cl_command_queue command_queue,
   POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (command_queue)),
                           CL_INVALID_COMMAND_QUEUE);
 
-  POCL_RETURN_ERROR_ON((command_queue->context->svm_allocdev == NULL),
-      CL_INVALID_CONTEXT, "None of the devices in this context is SVM-capable\n");
+  POCL_RETURN_ERROR_ON (
+      (command_queue->context->svm_allocdev == NULL), CL_INVALID_OPERATION,
+      "None of the devices in this context is SVM-capable\n");
 
   POCL_RETURN_ERROR_COND((svm_ptr == NULL), CL_INVALID_VALUE);
 
@@ -65,9 +66,6 @@ POname(clEnqueueSVMMemFill) (cl_command_queue command_queue,
   if (errcode != CL_SUCCESS)
     return errcode;
 
-  for(i=0; i<num_events_in_wait_list; i++)
-    POCL_RETURN_ERROR_COND((event_wait_list[i] == NULL), CL_INVALID_EVENT_WAIT_LIST);
-
   _cl_command_node *cmd = NULL;
   errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_SVM_MEMFILL,
                                  event, num_events_in_wait_list,
