@@ -37,23 +37,20 @@ POname(clEnqueueSVMUnmap) (cl_command_queue command_queue,
   POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (command_queue)),
                           CL_INVALID_COMMAND_QUEUE);
 
-  POCL_RETURN_ERROR_ON((command_queue->context->svm_allocdev == NULL),
-      CL_INVALID_CONTEXT, "None of the devices in this context is SVM-capable\n");
+  POCL_RETURN_ERROR_ON (
+      (command_queue->context->svm_allocdev == NULL), CL_INVALID_OPERATION,
+      "None of the devices in this context is SVM-capable\n");
 
-  if (DEVICE_MMAP_IS_NOP(command_queue->device)
-      && (num_events_in_wait_list == 0)
-      && (event == NULL))
-    return CL_SUCCESS;
-
-  POCL_RETURN_ERROR_COND((svm_ptr == NULL), CL_INVALID_VALUE);
+  POCL_RETURN_ERROR_COND ((svm_ptr == NULL), CL_INVALID_VALUE);
 
   errcode = pocl_check_event_wait_list (command_queue, num_events_in_wait_list,
                                         event_wait_list);
   if (errcode != CL_SUCCESS)
     return errcode;
 
-  for(i=0; i<num_events_in_wait_list; i++)
-    POCL_RETURN_ERROR_COND((event_wait_list[i] == NULL), CL_INVALID_EVENT_WAIT_LIST);
+  if (DEVICE_MMAP_IS_NOP (command_queue->device)
+      && (num_events_in_wait_list == 0) && (event == NULL))
+    return CL_SUCCESS;
 
   _cl_command_node *cmd = NULL;
 
