@@ -42,9 +42,11 @@ POname(clEnqueueWriteBuffer)(cl_command_queue command_queue,
   _cl_command_node *cmd = NULL;
   int errcode;
 
-  POCL_RETURN_ERROR_COND((command_queue == NULL), CL_INVALID_COMMAND_QUEUE);
+  POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (command_queue)),
+                          CL_INVALID_COMMAND_QUEUE);
 
-  POCL_RETURN_ERROR_COND((buffer == NULL), CL_INVALID_MEM_OBJECT);
+  POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (buffer)),
+                          CL_INVALID_MEM_OBJECT);
 
   POCL_RETURN_ON_SUB_MISALIGN (buffer, command_queue);
 
@@ -75,21 +77,22 @@ POname(clEnqueueWriteBuffer)(cl_command_queue command_queue,
 
   POCL_CHECK_DEV_IN_CMDQ;
 
-  errcode = pocl_create_command (&cmd, command_queue, 
-                                 CL_COMMAND_WRITE_BUFFER, 
-                                 event, num_events_in_wait_list, 
-                                 event_wait_list, 1, &buffer);
+  char rdonly = 0;
+
+  errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_WRITE_BUFFER,
+                                 event, num_events_in_wait_list,
+                                 event_wait_list, 1, &buffer, &rdonly);
   if (errcode != CL_SUCCESS)
-    return errcode;
+    {
+      POCL_MSG_ERR ("create command failed \n");
+      return errcode;
+    }
 
   cmd->command.write.src_host_ptr = ptr;
-  cmd->command.write.dst_mem_id = &buffer->device_ptrs[device->dev_id];
+  cmd->command.write.dst_mem_id = &buffer->device_ptrs[device->global_mem_id];
   cmd->command.write.offset = offset;
   cmd->command.write.size = size;
 
-  POname(clRetainMemObject) (buffer);
-  buffer->owning_device = command_queue->device;
-
   pocl_command_enqueue (command_queue, cmd);
 
   if (blocking_write)
