@@ -5,9 +5,9 @@
 /* Note - this is deprecated in 1.1, but (some of) the ICD loaders are built
  * against OCL 1.1, so we need it.
  */ 
-CL_API_ENTRY void * CL_API_CALL 
+POCL_EXPORT CL_API_ENTRY void * CL_API_CALL
 POname(clGetExtensionFunctionAddress)(const char * func_name ) 
-CL_EXT_SUFFIX__VERSION_1_0
+CL_API_SUFFIX__VERSION_1_0
 {
 
 #ifdef BUILD_ICD
