@@ -162,8 +162,8 @@ struct _cl_icd_dispatch pocl_dispatch = {
   NULL, /* &clUnknown122 */
 #if (OCL_ICD_IDENTIFIED_FUNCTIONS > 110)
   &POname(clCreateCommandQueueWithProperties),
-  NULL, /* &POname(clCreatePipe),*/
-  NULL, /* &POname(clGetPipeInfo),*/
+  &POname(clCreatePipe),
+  &POname(clGetPipeInfo),
   &POname(clSVMAlloc),
   &POname(clSVMFree),
   &POname(clEnqueueSVMFree),
@@ -171,22 +171,22 @@ struct _cl_icd_dispatch pocl_dispatch = {
   &POname(clEnqueueSVMMemFill),
   &POname(clEnqueueSVMMap),
   &POname(clEnqueueSVMUnmap),
-  NULL, /* clCreateSamplerWithProperties */
+  &POname(clCreateSamplerWithProperties),
   &POname(clSetKernelArgSVMPointer),
   &POname(clSetKernelExecInfo),
   NULL, /* clGetKernelSubGroupInfoKHR */
-  NULL, /* clCloneKernel */
+  &POname(clCloneKernel),
   &POname(clCreateProgramWithIL),
-  NULL, /* &clUnknown139 */
-  NULL, /* &clUnknown140 */
-  NULL, /* &clUnknown141 */
-  NULL, /* &clUnknown142 */
-  NULL, /* &clUnknown143 */
-  NULL, /* &clUnknown144 */
-  NULL, /* &clUnknown145 */
-  NULL, /* &clUnknown146 */
-  NULL, /* &clUnknown147 */
-  NULL, /* &clUnknown148 */
+  &POname(clEnqueueSVMMigrateMem),
+  &POname(clGetDeviceAndHostTimer),
+  &POname(clGetHostTimer),
+  &POname(clGetKernelSubGroupInfo),
+  &POname(clSetDefaultDeviceCommandQueue),
+  &POname(clSetProgramReleaseCallback),
+  &POname(clSetProgramSpecializationConstant),
+  &POname(clCreateBufferWithProperties),
+  &POname(clCreateImageWithProperties),
+  &POname(clSetContextDestructorCallback),
   NULL, /* &clUnknown149 */
   NULL, /* &clUnknown150 */
   NULL, /* &clUnknown151 */
