@@ -23,6 +23,7 @@
 */
 
 #include "basic.h"
+#include "builtin_kernels.hh"
 #include "common.h"
 #include "config.h"
 #include "config2.h"
@@ -107,6 +108,7 @@ pocl_basic_init_device_ops(struct pocl_device_ops *ops)
   ops->supports_binary = pocl_driver_supports_binary;
   ops->build_poclbinary = pocl_driver_build_poclbinary;
   ops->compile_kernel = pocl_basic_compile_kernel;
+  ops->build_builtin = pocl_driver_build_opencl_builtins;
 
   ops->join = pocl_basic_join;
   ops->submit = pocl_basic_submit;
@@ -143,13 +145,8 @@ char *
 pocl_basic_build_hash (cl_device_id device)
 {
   char* res = calloc(1000, sizeof(char));
-#ifdef KERNELLIB_HOST_DISTRO_VARIANTS
-  char *name = pocl_get_llvm_cpu_name ();
-  snprintf (res, 1000, "basic-%s-%s", HOST_DEVICE_BUILD_HASH, name);
-  POCL_MEM_FREE (name);
-#else
-  snprintf (res, 1000, "basic-%s", HOST_DEVICE_BUILD_HASH);
-#endif
+  snprintf (res, 1000, "basic-%s-%s", HOST_DEVICE_BUILD_HASH,
+            device->llvm_cpu);
   return res;
 }
 
@@ -205,8 +202,7 @@ pocl_basic_init (unsigned j, cl_device_id device, const char* parameters)
 
   pocl_setup_extensions_with_version (device);
 
-  /* builtin kernels.. skip, basic/pthread doesn't have any
-  pocl_setup_builtin_kernels_with_version (device); */
+  pocl_setup_builtin_kernels_with_version (device);
 
   pocl_setup_ils_with_version (device);
 
@@ -267,6 +263,8 @@ pocl_basic_init (unsigned j, cl_device_id device, const char* parameters)
      using multiple OpenCL devices. */
   device->max_compute_units = 1;
 
+  fix_local_mem_size (device);
+
   return ret;
 }
 
@@ -365,15 +363,30 @@ pocl_basic_run (void *data, _cl_command_node *cmd)
         }
     }
 
-  if (!cmd->device->device_alloca_locals)
-    for (i = 0; i < meta->num_locals; ++i)
-      {
-        size_t s = meta->local_sizes[i];
-        size_t j = meta->num_args + i;
-        arguments[j] = malloc (sizeof (void *));
-        void *pp = pocl_aligned_malloc (MAX_EXTENDED_ALIGNMENT, s);
-        *(void **)(arguments[j]) = pp;
-      }
+  if (cmd->device->device_alloca_locals)
+    {
+      /* Local buffers are allocated in the device side work-group
+         launcher. Let's pass only the sizes of the local args in
+         the arg buffer. */
+      for (i = 0; i < meta->num_locals; ++i)
+        {
+          assert (sizeof (size_t) == sizeof (void *));
+          size_t s = meta->local_sizes[i];
+          size_t j = meta->num_args + i;
+          *(size_t *)(arguments[j]) = s;
+        }
+    }
+  else
+    {
+      for (i = 0; i < meta->num_locals; ++i)
+        {
+          size_t s = meta->local_sizes[i];
+          size_t j = meta->num_args + i;
+          arguments[j] = malloc (sizeof (void *));
+          void *pp = pocl_aligned_malloc (MAX_EXTENDED_ALIGNMENT, s);
+          *(void **)(arguments[j]) = pp;
+        }
+    }
 
   pc->printf_buffer = d->printf_buffer;
   assert (pc->printf_buffer != NULL);
@@ -444,7 +457,7 @@ pocl_basic_run (void *data, _cl_command_node *cmd)
 void
 pocl_basic_run_native (void *data, _cl_command_node *cmd)
 {
-  cl_event ev = cmd->event;
+  cl_event ev = cmd->sync.event.event;
   cl_device_id dev = cmd->device;
   size_t i;
   for (i = 0; i < ev->num_buffers; i++)
@@ -500,8 +513,8 @@ static void basic_command_scheduler (struct data *d)
   /* execute commands from ready list */
   while ((node = d->ready_list))
     {
-      assert (pocl_command_is_ready(node->event));
-      assert (node->event->status == CL_SUBMITTED);
+      assert (pocl_command_is_ready (node->sync.event.event));
+      assert (node->sync.event.event->status == CL_SUBMITTED);
       CDL_DELETE (d->ready_list, node);
       POCL_UNLOCK (d->cq_lock);
       pocl_exec_command (node);
@@ -523,7 +536,7 @@ pocl_basic_submit (_cl_command_node *node, cl_command_queue cq)
   POCL_LOCK (d->cq_lock);
   pocl_command_push(node, &d->ready_list, &d->command_list);
 
-  POCL_UNLOCK_OBJ (node->event);
+  POCL_UNLOCK_OBJ (node->sync.event.event);
   basic_command_scheduler (d);
   POCL_UNLOCK (d->cq_lock);
 
@@ -585,8 +598,11 @@ void
 pocl_basic_compile_kernel (_cl_command_node *cmd, cl_kernel kernel,
                            cl_device_id device, int specialize)
 {
+  char *saved_name = NULL;
+  pocl_sanitize_builtin_kernel_name (kernel, &saved_name);
   if (cmd != NULL && cmd->type == CL_COMMAND_NDRANGE_KERNEL)
     pocl_check_kernel_dlhandle_cache (cmd, 0, specialize);
+  pocl_restore_builtin_kernel_name (kernel, saved_name);
 }
 
 /*********************** IMAGES ********************************/
