@@ -101,13 +101,8 @@ char *
 pocl_pthread_build_hash (cl_device_id device)
 {
   char* res = calloc(1000, sizeof(char));
-#ifdef KERNELLIB_HOST_DISTRO_VARIANTS
-  char *name = pocl_get_llvm_cpu_name ();
-  snprintf (res, 1000, "pthread-%s-%s", HOST_DEVICE_BUILD_HASH, name);
-  POCL_MEM_FREE (name);
-#else
-  snprintf (res, 1000, "pthread-%s", HOST_DEVICE_BUILD_HASH);
-#endif
+  snprintf (res, 1000, "pthread-%s-%s", HOST_DEVICE_BUILD_HASH,
+            device->llvm_cpu);
   return res;
 }
 
@@ -160,8 +155,7 @@ pocl_pthread_init (unsigned j, cl_device_id device, const char* parameters)
 
   pocl_setup_extensions_with_version (device);
 
-  /* builtin kernels.. skip, basic/pthread doesn't have any
-  pocl_setup_builtin_kernels_with_version (device); */
+  pocl_setup_builtin_kernels_with_version (device);
 
   pocl_setup_ils_with_version (device);
 
@@ -215,6 +209,8 @@ pocl_pthread_init (unsigned j, cl_device_id device, const char* parameters)
   pocl_cpuinfo_detect_device_info(device);
   pocl_set_buffer_image_limits(device);
 
+  fix_local_mem_size (device);
+
   /* in case hwloc doesn't provide a PCI ID, let's generate
      a vendor id that hopefully is unique across vendors. */
   const char *magic = "pocl";
@@ -297,12 +293,12 @@ void
 pocl_pthread_submit (_cl_command_node *node, cl_command_queue cq)
 {
   node->ready = 1;
-  if (pocl_command_is_ready (node->event))
+  if (pocl_command_is_ready (node->sync.event.event))
     {
-      pocl_update_event_submitted (node->event);
+      pocl_update_event_submitted (node->sync.event.event);
       pthread_scheduler_push_command (node);
     }
-  POCL_UNLOCK_OBJ (node->event);
+  POCL_UNLOCK_OBJ (node->sync.event.event);
   return;
 }
 
@@ -346,7 +342,7 @@ pocl_pthread_notify (cl_device_id device, cl_event event, cl_event finished)
   if (!node->ready)
     return;
 
-  if (pocl_command_is_ready (node->event))
+  if (pocl_command_is_ready (node->sync.event.event))
     {
       if (event->status == CL_QUEUED)
         {
@@ -387,6 +383,8 @@ pocl_pthread_update_event (cl_device_id device, cl_event event)
 
       PTHREAD_CHECK (pthread_cond_init (&e_d->event_cond, NULL));
       event->data = (void *) e_d;
+
+      VG_ASSOC_COND_VAR (e_d->event_cond, event->pocl_lock);
     }
 }
 
@@ -417,6 +415,11 @@ pocl_pthread_init_queue (cl_device_id device, cl_command_queue queue)
       = pocl_aligned_malloc (HOST_CPU_CACHELINE_SIZE, sizeof (pthread_cond_t));
   pthread_cond_t *cond = (pthread_cond_t *)queue->data;
   PTHREAD_CHECK (pthread_cond_init (cond, NULL));
+
+  POCL_LOCK_OBJ (queue);
+  VG_ASSOC_COND_VAR ((*cond), queue->pocl_lock);
+  POCL_UNLOCK_OBJ (queue);
+
   return CL_SUCCESS;
 }
 
