@@ -32,14 +32,16 @@
 #include <time.h>
 #include <unistd.h>
 
-#include "pocl-pthread_scheduler.h"
-#include "pocl_cl.h"
-#include "pocl-pthread.h"
-#include "pocl-pthread_utils.h"
-#include "utlist.h"
-#include "pocl_util.h"
+#include "builtin_kernels.hh"
 #include "common.h"
+#include "pocl-pthread.h"
+#include "pocl-pthread_scheduler.h"
+#include "pocl-pthread_utils.h"
+#include "pocl_cl.h"
 #include "pocl_mem_management.h"
+#include "pocl_util.h"
+#include "utlist.h"
+
 #ifdef __APPLE__
 #include "pthread_barrier.h"
 #endif
@@ -98,6 +100,10 @@ pthread_scheduler_init (cl_device_id device)
 
   PTHREAD_CHECK (pthread_cond_init (&(scheduler.wake_pool), NULL));
 
+  POCL_LOCK (scheduler.wq_lock_fast);
+  VG_ASSOC_COND_VAR (scheduler.wake_pool, scheduler.wq_lock_fast);
+  POCL_UNLOCK (scheduler.wq_lock_fast);
+
   scheduler.thread_pool = pocl_aligned_malloc (
       HOST_CPU_CACHELINE_SIZE,
       num_worker_threads * sizeof (struct pool_thread_data));
@@ -357,7 +363,8 @@ finalize_kernel_command (struct pool_thread_data *thread_data,
 
   pocl_release_dlhandle_cache (k->cmd);
 
-  POCL_UPDATE_EVENT_COMPLETE_MSG (k->cmd->event, "NDRange Kernel        ");
+  POCL_UPDATE_EVENT_COMPLETE_MSG (k->cmd->sync.event.event,
+                                  "NDRange Kernel        ");
 
   POCL_FAST_DESTROY (k->lock);
   free_kernel_run_command (k);
@@ -370,13 +377,19 @@ pocl_pthread_prepare_kernel (void *data, _cl_command_node *cmd)
   cl_kernel kernel = cmd->command.run.kernel;
   struct pocl_context *pc = &cmd->command.run.pc;
 
+  char *saved_name = NULL;
+  pocl_sanitize_builtin_kernel_name (kernel, &saved_name);
+  pocl_check_kernel_dlhandle_cache (cmd, 1, 1);
+  pocl_restore_builtin_kernel_name (kernel, saved_name);
+
   size_t num_groups = pc->num_groups[0] * pc->num_groups[1] * pc->num_groups[2];
 
   if (num_groups == 0)
     {
-      pocl_update_event_running (cmd->event);
+      pocl_update_event_running (cmd->sync.event.event);
 
-      POCL_UPDATE_EVENT_COMPLETE_MSG (cmd->event, "NDRange Kernel        ");
+      POCL_UPDATE_EVENT_COMPLETE_MSG (cmd->sync.event.event,
+                                      "NDRange Kernel        ");
 
       return;
     }
@@ -402,7 +415,7 @@ pocl_pthread_prepare_kernel (void *data, _cl_command_node *cmd)
 
   setup_kernel_arg_array (run_cmd);
 
-  pocl_update_event_running (cmd->event);
+  pocl_update_event_running (cmd->sync.event.event);
 
   pthread_scheduler_push_kernel (run_cmd);
 }
@@ -497,7 +510,7 @@ RETRY:
     {
       POCL_FAST_UNLOCK (scheduler.wq_lock_fast);
 
-      assert (pocl_command_is_ready (cmd->event));
+      assert (pocl_command_is_ready (cmd->sync.event.event));
 
       if (cmd->type == CL_COMMAND_NDRANGE_KERNEL)
         {
