@@ -124,13 +124,15 @@ pocl_check_device_supports_image (cl_device_id device,
                                   const cl_image_format *image_format,
                                   const cl_image_desc *image_desc,
                                   cl_uint image_type_idx,
-                                  int *device_support)
+                                  cl_bool is_gl_texture, int *device_support)
 {
   cl_uint i;
   size_t m;
 
-  if (device_support)
-    *device_support = 0;
+  *device_support = 0;
+
+  if (device->has_gl_interop || (!is_gl_texture))
+    *device_support |= DEVICE_IMAGE_INTEROP_SUPPORT;
 
   POCL_RETURN_ERROR_ON((!device->image_support), CL_INVALID_OPERATION,
           "Device does not support images");
@@ -186,8 +188,7 @@ pocl_check_device_supports_image (cl_device_id device,
           "Image buffer size (width) > device.max_buffer_size\n");
     }
 
-  if (device_support)
-    *device_support |= DEVICE_IMAGE_SIZE_SUPPORT;
+  *device_support |= DEVICE_IMAGE_SIZE_SUPPORT;
 
   for (i = 0; i < device->num_image_formats[image_type_idx]; i++)
     {
@@ -198,8 +199,7 @@ pocl_check_device_supports_image (cl_device_id device,
           && p[i].image_channel_data_type
                  == image_format->image_channel_data_type)
         {
-          if (device_support)
-            *device_support |= DEVICE_IMAGE_FORMAT_SUPPORT;
+          *device_support |= DEVICE_IMAGE_FORMAT_SUPPORT;
 
           return CL_SUCCESS;
         }
@@ -558,6 +558,16 @@ pocl_write_pixel_fast_ui (cl_uint4 color, int order, int elem_size, void *data)
         *((cl_uint *)data) = color.s[3];
       return;
     }
+  if (order == CL_R)
+    {
+      if (elem_size == 1)
+        *((cl_uchar *)data) = convert_uchar_sat_int (color.s[0]);
+      else if (elem_size == 2)
+        *((cl_ushort *)data) = convert_ushort_sat_int (color.s[0]);
+      else if (elem_size == 4)
+        *((cl_uint *)data) = color.s[0];
+      return;
+    }
 
   if (elem_size == 1)
     {
@@ -586,6 +596,10 @@ pocl_write_pixel_fast_f (cl_float4 color, int channel_type, int order,
     {
       write_float_pixel (color.s[3], data, channel_type);
     }
+  else if (order == CL_R)
+    {
+      write_float_pixel (color.s[0], data, channel_type);
+    }
   else
     {
       write_float4_pixel (color, data, channel_type);
@@ -611,6 +625,17 @@ pocl_write_pixel_fast_i (cl_int4 color, int order, int elem_size, void *data)
       return;
     }
 
+  if (order == CL_R)
+    {
+      if (elem_size == 1)
+        *((cl_char *)data) = convert_char_sat_int (color.s[0]);
+      else if (elem_size == 2)
+        *((cl_short *)data) = convert_short_sat_int (color.s[0]);
+      else if (elem_size == 4)
+        *((cl_int *)data) = color.s[0];
+      return;
+    }
+
   if (elem_size == 1)
     {
       *((cl_char4 *)data) = convert_char4_sat_int (color);
