@@ -32,6 +32,7 @@
 
 void origin_to_bytes (cl_mem mem, const size_t *origin, size_t *byte_offset);
 
+POCL_EXPORT
 cl_int opencl_image_type_to_index (cl_mem_object_type  image_type);
 
 extern cl_int 
@@ -45,11 +46,10 @@ pocl_get_image_information (cl_channel_order  ch_order,
                             int*           host_channels,
                             int*           host_elem_size);
 
-extern cl_int
-pocl_check_device_supports_image (cl_device_id device,
-                                  const cl_image_format *image_format,
-                                  const cl_image_desc *image_desc,
-                                  cl_uint image_type_idx, int *device_support);
+extern cl_int pocl_check_device_supports_image (
+    cl_device_id device, const cl_image_format *image_format,
+    const cl_image_desc *image_desc, cl_uint image_type_idx,
+    cl_bool is_gl_texture, int *device_support);
 
 void pocl_write_pixel_zero (void *data, const cl_uint4 input_color, int order,
                             int elem_size, int channel_type);
