@@ -145,15 +145,9 @@ static TargetMachine *GetTargetMachine(cl_device_id device, Triple &triple) {
     return 0;
   }
 
-#ifdef LLVM_OLDER_THAN_6_0
-  TargetMachine *TM = TheTarget->createTargetMachine(
-      triple.getTriple(), MCPU, StringRef(""), GetTargetOptions(), Reloc::PIC_,
-      CodeModel::Default, CodeGenOpt::Aggressive);
-#else
   TargetMachine *TM = TheTarget->createTargetMachine(
       triple.getTriple(), MCPU, StringRef(""), GetTargetOptions(), Reloc::PIC_,
       CodeModel::Small, CodeGenOpt::Aggressive);
-#endif
 
   assert(TM != NULL && "llvm target has no targetMachine constructor");
   if (device->ops->init_target_machine)
