@@ -26,12 +26,15 @@
 
 #include "config.h"
 
+#include "pocl_export.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 extern const unsigned pocl_timer_resolution;
 
+POCL_EXPORT
 uint64_t pocl_gettimemono_ns();
 
 int pocl_gettimereal(int *year, int *mon, int *day, int *hour, int *min, int *sec, int* nanosec);
