@@ -114,6 +114,7 @@ double Sleef_exp2_u10 (double);
 double Sleef_exp10_u10 (double);
 double Sleef_expm1_u10 (double);
 double Sleef_log10_u10 (double);
+double Sleef_log2_u10 (double);
 double Sleef_log1p_u10 (double);
 Sleef_double2 Sleef_sincospi_u05 (double);
 Sleef_double2 Sleef_sincospi_u35 (double);
@@ -122,7 +123,9 @@ double Sleef_cospi_u05 (double);
 double Sleef_ldexp (double, int);
 int Sleef_ilogb (double);
 double Sleef_fma (double, double, double);
+double Sleef_sqrt (double);
 double Sleef_sqrt_u05 (double);
+double Sleef_sqrt_u35 (double);
 double Sleef_hypot_u05 (double, double);
 double Sleef_hypot_u35 (double, double);
 double Sleef_fabs (double);
@@ -191,6 +194,7 @@ float Sleef_exp2f_u10 (float);
 float Sleef_exp10f_u10 (float);
 float Sleef_expm1f_u10 (float);
 float Sleef_log10f_u10 (float);
+float Sleef_log2f_u10 (float);
 float Sleef_log1pf_u10 (float);
 Sleef_float2 Sleef_sincospif_u05 (float);
 Sleef_float2 Sleef_sincospif_u35 (float);
@@ -199,6 +203,7 @@ float Sleef_cospif_u05 (float d);
 float Sleef_ldexpf (float, int);
 int Sleef_ilogbf (float);
 float Sleef_fmaf (float, float, float);
+float Sleef_sqrtf (float);
 float Sleef_sqrtf_u05 (float);
 float Sleef_sqrtf_u35 (float);
 float Sleef_hypotf_u05 (float, float);
@@ -274,6 +279,7 @@ double8 Sleef_exp2d8_u10 (double8);
 double8 Sleef_exp10d8_u10 (double8);
 double8 Sleef_expm1d8_u10 (double8);
 double8 Sleef_log10d8_u10 (double8);
+double8 Sleef_log2d8_u10 (double8);
 double8 Sleef_log1pd8_u10 (double8);
 Sleef_double8_2 Sleef_sincospid8_u05 (double8);
 Sleef_double8_2 Sleef_sincospid8_u35 (double8);
@@ -282,6 +288,7 @@ double8 Sleef_cospid8_u05 (double8);
 double8 Sleef_ldexpd8 (double8, int8);
 int8 Sleef_ilogbd8 (double8);
 double8 Sleef_fmad8 (double8, double8, double8);
+double8 Sleef_sqrtd8 (double8);
 double8 Sleef_sqrtd8_u05 (double8);
 double8 Sleef_sqrtd8_u35 (double8);
 double8 Sleef_hypotd8_u05 (double8, double8);
@@ -349,6 +356,7 @@ float16 Sleef_exp2f16_u10 (float16);
 float16 Sleef_exp10f16_u10 (float16);
 float16 Sleef_expm1f16_u10 (float16);
 float16 Sleef_log10f16_u10 (float16);
+float16 Sleef_log2f16_u10 (float16);
 float16 Sleef_log1pf16_u10 (float16);
 Sleef_float16_2 Sleef_sincospif16_u05 (float16);
 Sleef_float16_2 Sleef_sincospif16_u35 (float16);
@@ -357,6 +365,7 @@ float16 Sleef_cospif16_u05 (float16);
 float16 Sleef_ldexpf16 (float16, int16);
 int16 Sleef_ilogbf16 (float16);
 float16 Sleef_fmaf16 (float16, float16, float16);
+float16 Sleef_sqrtf16 (float16);
 float16 Sleef_sqrtf16_u05 (float16);
 float16 Sleef_sqrtf16_u35 (float16);
 float16 Sleef_hypotf16_u05 (float16, float16);
@@ -436,6 +445,7 @@ double4 Sleef_exp2d4_u10 (double4);
 double4 Sleef_exp10d4_u10 (double4);
 double4 Sleef_expm1d4_u10 (double4);
 double4 Sleef_log10d4_u10 (double4);
+double4 Sleef_log2d4_u10 (double4);
 double4 Sleef_log1pd4_u10 (double4);
 Sleef_double4_2 Sleef_sincospid4_u05 (double4);
 Sleef_double4_2 Sleef_sincospid4_u35 (double4);
@@ -444,6 +454,7 @@ double4 Sleef_cospid4_u05 (double4);
 double4 Sleef_ldexpd4 (double4, int4);
 int4 Sleef_ilogbd4 (double4);
 double4 Sleef_fmad4 (double4, double4, double4);
+double4 Sleef_sqrtd4 (double4);
 double4 Sleef_sqrtd4_u05 (double4);
 double4 Sleef_sqrtd4_u35 (double4);
 double4 Sleef_hypotd4_u05 (double4, double4);
@@ -511,6 +522,7 @@ float8 Sleef_exp2f8_u10 (float8);
 float8 Sleef_exp10f8_u10 (float8);
 float8 Sleef_expm1f8_u10 (float8);
 float8 Sleef_log10f8_u10 (float8);
+float8 Sleef_log2f8_u10 (float8);
 float8 Sleef_log1pf8_u10 (float8);
 Sleef_float8_2 Sleef_sincospif8_u05 (float8);
 Sleef_float8_2 Sleef_sincospif8_u35 (float8);
@@ -519,6 +531,7 @@ float8 Sleef_cospif8_u05 (float8);
 float8 Sleef_ldexpf8 (float8, int8);
 int8 Sleef_ilogbf8 (float8);
 float8 Sleef_fmaf8 (float8, float8, float8);
+float8 Sleef_sqrtf8 (float8);
 float8 Sleef_sqrtf8_u05 (float8);
 float8 Sleef_sqrtf8_u35 (float8);
 float8 Sleef_hypotf8_u05 (float8, float8);
@@ -595,6 +608,7 @@ double2 Sleef_exp2d2_u10 (double2);
 double2 Sleef_exp10d2_u10 (double2);
 double2 Sleef_expm1d2_u10 (double2);
 double2 Sleef_log10d2_u10 (double2);
+double2 Sleef_log2d2_u10 (double2);
 double2 Sleef_log1pd2_u10 (double2);
 Sleef_double2_2 Sleef_sincospid2_u05 (double2);
 Sleef_double2_2 Sleef_sincospid2_u35 (double2);
@@ -603,6 +617,7 @@ double2 Sleef_cospid2_u05 (double2);
 double2 Sleef_ldexpd2 (double2, int2);
 int2 Sleef_ilogbd2 (double2);
 double2 Sleef_fmad2 (double2, double2, double2);
+double2 Sleef_sqrtd2 (double2);
 double2 Sleef_sqrtd2_u05 (double2);
 double2 Sleef_sqrtd2_u35 (double2);
 double2 Sleef_hypotd2_u05 (double2, double2);
@@ -673,6 +688,7 @@ float4 Sleef_exp2f4_u10 (float4);
 float4 Sleef_exp10f4_u10 (float4);
 float4 Sleef_expm1f4_u10 (float4);
 float4 Sleef_log10f4_u10 (float4);
+float4 Sleef_log2f4_u10 (float4);
 float4 Sleef_log1pf4_u10 (float4);
 Sleef_float4_2 Sleef_sincospif4_u05 (float4);
 Sleef_float4_2 Sleef_sincospif4_u35 (float4);
@@ -681,6 +697,7 @@ float4 Sleef_cospif4_u05 (float4);
 float4 Sleef_ldexpf4 (float4, int4);
 int4 Sleef_ilogbf4 (float4);
 float4 Sleef_fmaf4 (float4, float4, float4);
+float4 Sleef_sqrtf4 (float4);
 float4 Sleef_sqrtf4_u05 (float4);
 float4 Sleef_sqrtf4_u35 (float4);
 float4 Sleef_hypotf4_u05 (float4, float4);
