@@ -194,7 +194,7 @@ void dumpCFG(
   std::cout << "### dumped CFG to " << fname << std::endl;
 }
 
-bool chopBBs(llvm::Function &F, llvm::Pass &P) {
+bool chopBBs(llvm::Function &F, llvm::Pass &) {
   bool fchanged = false;
   const int MAX_INSTRUCTIONS_PER_BB = 70;
   do {
@@ -211,11 +211,7 @@ bool chopBBs(llvm::Function &F, llvm::Pass &P) {
               ++splitPoint;
               ++count;
             }
-#ifdef LLVM_OLDER_THAN_3_7
-          SplitBlock(b, splitPoint, &P);
-#else
           SplitBlock(b, &*splitPoint);
-#endif
           fchanged = true;
           break;
         }
@@ -224,5 +220,4 @@ bool chopBBs(llvm::Function &F, llvm::Pass &P) {
   } while (fchanged);
   return fchanged;
 }
-    
 };
