@@ -755,10 +755,8 @@ WorkitemLoops::GetLinearWiIndex(llvm::IRBuilder<> &builder, llvm::Module *M,
                              "linear_xyz_idx");
 }
 
-llvm::Instruction *
-WorkitemLoops::AddContextSave
-(llvm::Instruction *instruction, llvm::Instruction *alloca)
-{
+llvm::Instruction *WorkitemLoops::AddContextSave(llvm::Instruction *instruction,
+                                                 llvm::AllocaInst *alloca) {
 
   if (isa<AllocaInst>(instruction))
     {
@@ -776,7 +774,7 @@ WorkitemLoops::AddContextSave
 
   IRBuilder<> builder(&*definition);
   std::vector<llvm::Value *> gepArgs;
-  
+
   /* Reuse the id loads earlier in the region, if possible, to
      avoid messy output with lots of redundant loads. */
   ParallelRegion *region = RegionOfBlock(instruction->getParent());
@@ -796,33 +794,31 @@ WorkitemLoops::AddContextSave
       gepArgs.push_back(region->LocalIDXLoad());
     }
 
-  return builder.CreateStore(instruction,
-             builder.CreateGEP(alloca->getType()->getPointerElementType(),
-                               alloca, gepArgs));
+    return builder.CreateStore(
+        instruction,
+#ifdef LLVM_OLDER_THAN_15_0
+        builder.CreateGEP(alloca->getType()->getPointerElementType(), alloca,
+                          gepArgs));
+#else
+        builder.CreateGEP(alloca->getAllocatedType(), alloca, gepArgs));
+#endif
 }
 
-llvm::Instruction *WorkitemLoops::AddContextRestore(llvm::Value *val,
-                                                    llvm::Instruction *alloca,
-                                                    llvm::Type *InstType,
-                                                    bool PoclWrapperStructAdded,
-                                                    llvm::Instruction *before,
-                                                    bool isAlloca) {
-  assert (val != NULL);
-  assert (alloca != NULL);
+llvm::Instruction *WorkitemLoops::AddContextRestore(
+    llvm::Value *val, llvm::AllocaInst *alloca, llvm::Type *InstType,
+    bool PoclWrapperStructAdded, llvm::Instruction *before, bool isAlloca) {
+
+  assert(val != NULL);
+  assert(alloca != NULL);
   IRBuilder<> builder(alloca);
-  if (before != NULL) 
-    {
-      builder.SetInsertPoint(before);
-    }
-  else if (isa<Instruction>(val))
-    {
-      builder.SetInsertPoint(dyn_cast<Instruction>(val));
-      before = dyn_cast<Instruction>(val);
-    }
-  else 
-    {
-      assert (false && "Unknown context restore location!");
-    }
+  if (before != NULL) {
+    builder.SetInsertPoint(before);
+  } else if (isa<Instruction>(val)) {
+    builder.SetInsertPoint(dyn_cast<Instruction>(val));
+    before = dyn_cast<Instruction>(val);
+  } else {
+    assert(false && "Unknown context restore location!");
+  }
 
   std::vector<llvm::Value *> gepArgs;
 
@@ -849,10 +845,13 @@ llvm::Instruction *WorkitemLoops::AddContextRestore(llvm::Value *val,
       gepArgs.push_back(
           ConstantInt::get(Type::getInt32Ty(alloca->getContext()), 0));
 
-    llvm::Instruction *gep =
-        dyn_cast<Instruction>(builder.CreateGEP(
-            alloca->getType()->getPointerElementType(),
-            alloca, gepArgs));
+    llvm::Instruction *gep = dyn_cast<Instruction>(builder.CreateGEP(
+#ifdef LLVM_OLDER_THAN_15_0
+        alloca->getType()->getPointerElementType(), alloca, gepArgs));
+#else
+        alloca->getAllocatedType(), alloca, gepArgs));
+#endif
+
     if (isAlloca) {
       /* In case the context saved instruction was an alloca, we created a
          context array with pointed-to elements, and now want to return a
@@ -866,9 +865,8 @@ llvm::Instruction *WorkitemLoops::AddContextRestore(llvm::Value *val,
  * Returns the context array (alloca) for the given Value, creates it if not
  * found.
  */
-llvm::Instruction *
-WorkitemLoops::GetContextArray(llvm::Instruction *instruction,
-                               bool &PoclWrapperStructAdded) {
+llvm::AllocaInst *WorkitemLoops::GetContextArray(llvm::Instruction *instruction,
+                                                 bool &PoclWrapperStructAdded) {
   PoclWrapperStructAdded = false;
   /*
    * Unnamed temp instructions need a generated name for the
@@ -901,7 +899,6 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction,
   IRBuilder<> builder(&*(bb.getFirstInsertionPt()));
   Function *FF = instruction->getParent()->getParent();
   Module *M = instruction->getParent()->getParent()->getParent();
-  LLVMContext &C = M->getContext();
   const llvm::DataLayout &Layout = M->getDataLayout();
   DICompileUnit *CU = nullptr;
   std::unique_ptr<DIBuilder> DB;
@@ -958,7 +955,7 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction,
          unique stack space to all the work-items when its wiloop
          iteration is executed. */
       elementType = 
-        dyn_cast<AllocaInst>(instruction)->getType()->getElementType();
+        dyn_cast<AllocaInst>(instruction)->getAllocatedType();
     } 
   else 
     {
@@ -973,8 +970,11 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction,
   Type *AllocType = elementType;
   AllocaInst *InstCast = dyn_cast<AllocaInst>(instruction);
   if (InstCast) {
+#ifdef LLVM_OLDER_THAN_15_0
     unsigned Alignment = InstCast->getAlignment();
-
+#else
+    unsigned Alignment = InstCast->getAlign().value();
+#endif
     uint64_t StoreSize =
         Layout.getTypeStoreSize(InstCast->getAllocatedType());
 
@@ -1079,18 +1079,27 @@ WorkitemLoops::GetContextArray(llvm::Instruction *instruction,
 
       size_t sizeBits;
 #ifndef LLVM_OLDER_THAN_7_0
-      sizeBits = Alloca->getAllocationSizeInBits(M->getDataLayout())
-#ifndef LLVM_OLDER_THAN_12_0
+      sizeBits = Alloca
+                     ->getAllocationSizeInBits(M->getDataLayout())
+#if !defined(LLVM_OLDER_THAN_15_0)
+                     .value_or(TypeSize(0, false))
+                     .getFixedValue();
+#elif !defined(LLVM_OLDER_THAN_12_0)
                      .getValueOr(TypeSize(0, false))
                      .getFixedValue();
 #else
                      .getValueOr(0);
 #endif
+
       assert(sizeBits != 0);
 #endif
       // if (size == 0) WGLocalSizeX * WGLocalSizeY * WGLocalSizeZ * 8 *
       // Alloca->getAllocatedType()->getScalarSizeInBits();
+#ifdef LLVM_OLDER_THAN_15_0
       size_t alignBits = Alloca->getAlignment() * 8;
+#else
+      size_t alignBits = Alloca->getAlign().value() * 8;
+#endif
 
       Metadata *VariableDebugMeta =
           cast<MetadataAsValue>(DebugCall->getOperand(1))->getMetadata();
@@ -1159,7 +1168,7 @@ WorkitemLoops::AddContextSaveRestore
 
   /* Allocate the context data array for the variable. */
   bool PoclWrapperStructAdded = false;
-  llvm::Instruction *alloca =
+  llvm::AllocaInst *alloca =
       GetContextArray(instruction, PoclWrapperStructAdded);
   llvm::Instruction *theStore = AddContextSave(instruction, alloca);
 
