@@ -17,10 +17,12 @@
  *
  * log is used to report errors if we run into undefined symbols
  */
-int link(llvm::Module *krn, const llvm::Module *lib, std::string &log);
+int link(llvm::Module *krn, const llvm::Module *lib, std::string &log,
+         unsigned global_AS, const char **DevAuxFuncs);
 
 int copyKernelFromBitcode(const char *name, llvm::Module *parallel_bc,
-                          const llvm::Module *program);
+                          const llvm::Module *program, unsigned global_AS,
+                          const char **DevAuxFuncs);
 
 #ifdef __GNUC__
 #pragma GCC visibility pop
