@@ -1,11 +1,12 @@
-
 // Used to cause an LLVM crash with Haswell/Broadwell.
 // See https://github.com/pocl/pocl/issues/231
 
+#include "pocl_opencl.h"
+
 #define CL_HPP_ENABLE_EXCEPTIONS
 #define CL_HPP_MINIMUM_OPENCL_VERSION 120
 #define CL_HPP_TARGET_OPENCL_VERSION 120
-#include <CL/cl2.hpp>
+#include <CL/opencl.hpp>
 #include <iostream>
 
 using namespace std;
@@ -100,4 +101,7 @@ int main(int argc, char *argv[])
 
   queue.finish();
   cl::Platform::getDefault().unloadCompiler();
+
+  std::cout << "OK" << std::endl;
+  return EXIT_SUCCESS;
 }
