@@ -1,10 +1,11 @@
-
 // See https://github.com/pocl/pocl/issues/553
 
+#include "pocl_opencl.h"
+
 #define CL_HPP_ENABLE_EXCEPTIONS
 #define CL_HPP_MINIMUM_OPENCL_VERSION 120
 #define CL_HPP_TARGET_OPENCL_VERSION 120
-#include <CL/cl2.hpp>
+#include <CL/opencl.hpp>
 #include <iostream>
 
 using namespace std;
@@ -72,4 +73,7 @@ int main(int argc, char *argv[])
 
   queue.finish();
   cl::Platform::getDefault().unloadCompiler();
+
+  std::cout << "OK" << std::endl;
+  return EXIT_SUCCESS;
 }
