@@ -1,7 +1,7 @@
 /* Tests a case where a program created from binary (with a loop) is run with
    a local size of (1, 1, 1).
 
-   Copyright (c) 2017 pocl developers
+   Copyright (c) 2017-2021 pocl developers
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
@@ -22,8 +22,8 @@
    THE SOFTWARE.
 */
 
-#include "poclu.h"
-#include <CL/opencl.h>
+#include "pocl_opencl.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -82,6 +82,10 @@ int main ()
         }
     }
 
+  /* Test executing from the generic binary only to avoid the
+     online compiler creating a specialized one that is ran. */
+  setenv("POCL_WORK_GROUP_SPECIALIZATION", "0", 1);
+
   err = clGetPlatformIDs(1, &platform, NULL);
   CHECK_CL_ERROR (err);
   err = clGetDeviceIDs(platform, CL_DEVICE_TYPE_DEFAULT, 1, &device, NULL);
@@ -132,7 +136,7 @@ int main ()
             {
               printf("Error at %u %u : %u != %u\n", k, i,
                      output_buffer[k*vec_size+i], expected);
-              return 1;
+              return EXIT_FAILURE;
             }
         }
     }
@@ -145,5 +149,7 @@ int main ()
   clReleaseContext(context);
   clReleaseDevice(device);
   clUnloadPlatformCompiler(platform);
-  return 0;
+
+  printf ("OK\n");
+  return EXIT_SUCCESS;
 }
