@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "config.h"
 #include "poclu.h"
 
 /*
@@ -64,8 +65,9 @@ main (int argc, char **argv)
       goto EARLY_EXIT;
     }
 
+  const char *sourcefile = SRCDIR "/tests/runtime/migration_test";
   const char *basename = "migration_test";
-  err = poclu_load_program_multidev (context, devices, num_devices, basename,
+  err = poclu_load_program_multidev (context, devices, num_devices, sourcefile,
                                      0, 0, 0, NULL, NULL, &program);
   if (err != CL_SUCCESS)
     goto ERROR;
