@@ -25,8 +25,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <CL/cl.h>
-#include <poclu.h>
+
+#include "poclu.h"
 #include "config.h"
 
 #define MAX_PLATFORMS 32
